@@ -2,9 +2,9 @@
 
 {- |
   Module      :  Codec.Binary.UTF8.Light
-  Copyright   :  (c) Matt Morrow 2008
+  Copyright   :  (c) Matt Morrow 2008, Francesco Ariis 2022
   License     :  BSD3
-  Maintainer  :  Matt Morrow <mjm2002@gmail.com>
+  Maintainer  :  Francesco Ariis <fa-ml@ariis.it>
   Stability   :  provisional
   Portability :  portable
 
@@ -31,10 +31,6 @@ module Codec.Binary.UTF8.Light (
   , hGetUTF8Contents
   , hGetUTF8
   , hGetUTF8NonBlocking
-  , c2w
-  , w2c
-  , i2w
-  , w2i
   , flipUTF8
   , unflipUTF8
   , flipTab
@@ -44,36 +40,28 @@ module Codec.Binary.UTF8.Light (
   , fromBits
   , Int8,Int16,Int32
   , Word,Word8,Word16,Word32
+  , c2w, w2c
 ) where
 
 import Data.Bits
 import Data.List(foldl')
 import Data.Char(chr,ord)
-import Data.Monoid(Monoid(..))
 import Data.ByteString(ByteString)
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as B8
 import qualified Data.ByteString.Internal as B
 import Data.ByteString.Unsafe
 import System.IO(Handle)
+import Codec.Binary.UTF8.Light.Helper (c2w, w2c, cwrd, wh, toW8)
 
 #if defined(__GLASGOW_HASKELL__)
 import GHC.Exts
-  (Int(I#),Word(W#),Char(C#)
-  ,Ptr(Ptr),FunPtr(FunPtr))
-import GHC.Int
-  (Int8(I8#),Int16(I16#),Int32(I32#))
+  (Int(I#))
+import GHC.Int (Int8, Int16, Int32)
 import GHC.Word
-  (Word8(W8#),Word16(W16#),Word32(W32#))
+  (Word8, Word16, Word32(W32#))
 import GHC.Prim
-  (Char#,Int#,Word#,Addr#
-  ,ord#,chr#,int2Word#,word2Int#
-  ,and#,or#,xor#,not#
-  ,gtWord#,geWord#,eqWord#
-  ,neWord#,ltWord#,leWord#
-  ,uncheckedShiftL#,uncheckedShiftRL#
-  ,narrow8Int#,narrow16Int#,narrow32Int#
-  ,narrow8Word#,narrow16Word#,narrow32Word#)
+  (int2Word# ,and#, or#, ltWord#, uncheckedShiftRL#)
 #else
 import Data.Word
   (Word,Word8,Word16,Word32)
@@ -176,7 +164,7 @@ lenUTF16 w16
 
 -- | Lengths in Word8s
 countUTF8 :: ByteString -> [Int]
-countUTF8 s = go 0 (B.length s) s
+countUTF8 bs = go 0 (B.length bs) bs
   where go :: Int -> Int -> ByteString -> [Int]
         go i len s | len <= i = []
           | otherwise = case lenUTF8 (unsafeIndex s i)
@@ -244,6 +232,7 @@ decodeUTF8 s = go 0 (B.length s) s
 
 #else
 
+
 -- | Word32s not representing
 --  valid UTF8 chars are /dropped/.
 encodeUTF8' :: [Word32] -> [[Word8]]
@@ -253,55 +242,59 @@ encodeUTF8' [] = []
 -- ==> 0xff00ff00##
 encodeUTF8' ((W32# w):xs)
 #if MIN_VERSION_base(4,7,0)
-  | I# (w`ltWord#`(int2Word# 0x80#)) /= 0 =
+  | I# (ww`ltWord#`(int2Word# 0x80#)) /= 0 =
 #else
   | w`ltWord#`(int2Word# 0x80#) =
 #endif
-      [W8# w] : encodeUTF8' xs
+      [wh w] : encodeUTF8' xs
+
 #if MIN_VERSION_base(4,7,0)
-  | I# (w`ltWord#`(int2Word# 0x800#)) /= 0 =
+  | I# (ww`ltWord#`(int2Word# 0x800#)) /= 0 =
 #else
   | w`ltWord#`(int2Word# 0x800#) =
 #endif
-      [ W8#(w`uncheckedShiftRL#`6#
+      [ toW8(ww`uncheckedShiftRL#`6#
               `or#`(int2Word# 0xc0#))
-      , W8#(w`and#`(int2Word# 0x3f#)
+      , toW8(ww`and#`(int2Word# 0x3f#)
               `or#`(int2Word# 0x80#))
       ] : encodeUTF8' xs
 #if MIN_VERSION_base(4,7,0)
-  | I# (w`ltWord#`(int2Word# 0xf0000#)) /= 0 =
+  | I# (ww`ltWord#`(int2Word# 0xf0000#)) /= 0 =
 #else
   | w`ltWord#`(int2Word# 0xf0000#) =
 #endif
-      [ W8#(w`uncheckedShiftRL#`12#
+      [ toW8(ww`uncheckedShiftRL#`12#
               `or#`(int2Word# 0xe0#))
-      , W8#(w`uncheckedShiftRL#`6#
+      , toW8(ww`uncheckedShiftRL#`6#
               `and#`(int2Word# 0x3f#)
                 `or#`(int2Word# 0x80#))
-      , W8#(w`and#`(int2Word# 0x3f#)
+      , toW8(ww`and#`(int2Word# 0x3f#)
               `or#`(int2Word# 0x80#))
       ] : encodeUTF8' xs
 #if MIN_VERSION_base(4,7,0)
-  | I# (w`ltWord#`(int2Word# 0xe00000#)) /= 0 =
+  | I# (ww`ltWord#`(int2Word# 0xe00000#)) /= 0 =
 #else
   | w`ltWord#`(int2Word# 0xe00000#) =
 #endif
-      [ W8#(w`uncheckedShiftRL#`18#
+      [ toW8(ww`uncheckedShiftRL#`18#
               `or#`(int2Word# 0xf0#))
-      , W8#(w`uncheckedShiftRL#`12#
+      , toW8(ww`uncheckedShiftRL#`12#
               `and#`(int2Word# 0x3f#)
                 `or#`(int2Word# 0x80#))
-      , W8#(w`uncheckedShiftRL#`6#
+      , toW8(ww`uncheckedShiftRL#`6#
               `and#`(int2Word# 0x3f#)
                 `or#`(int2Word# 0x80#))
-      , W8#(w`and#`(int2Word# 0x3f#)
+      , toW8(ww`and#`(int2Word# 0x3f#)
               `or#`(int2Word# 0x80#))
       ] : encodeUTF8' xs
   | otherwise = [] : encodeUTF8' xs
 
+    where
+          ww = cwrd w
+
 -- TODO: ghc-ify decodeUTF8
 decodeUTF8 :: ByteString -> [Word32]
-decodeUTF8 s = go 0 (B.length s) s
+decodeUTF8 bs = go 0 (B.length bs) bs
   where go :: Int -> Int -> ByteString -> [Word32]
         go i len s | len <= i  = []
           | otherwise = let c1 = unsafeIndex s i
@@ -329,42 +322,10 @@ decodeUTF8 s = go 0 (B.length s) s
                                         `xor`fi(c3.&.0x3f)`shiftL`6
                                           `xor`fi(c4.&.0x3f)
                                             : go (i+4) len s
+                            _ -> error "decodeUTF8: len > 4"
 
 #endif
 
------------------------------------------------------------------------------
-
-w2c :: Word32 -> Char
-{-# INLINE w2c #-}
-#if defined(__GLASGOW_HASKELL__)
-w2c (W32# w) = C#(chr#(word2Int# w))
-#else
-w2c = unsafeChr . fromIntegral
-#endif
-
-c2w :: Char -> Word32
-{-# INLINE c2w #-}
-#if defined(__GLASGOW_HASKELL__)
-c2w (C# c) = W32#(int2Word#(ord# c))
-#else
-c2w = fromIntegral . ord
-#endif
-
-i2w :: Int -> Word32
-{-# INLINE i2w #-}
-#if defined(__GLASGOW_HASKELL__)
-i2w (I# i) = W32#(int2Word# i)
-#else
-i2w = fi
-#endif
-
-w2i :: Word32 -> Int
-{-# INLINE w2i #-}
-#if defined(__GLASGOW_HASKELL__)
-w2i (W32# w) = I#(word2Int# w)
-#else
-w2i = fi
-#endif
 
 -----------------------------------------------------------------------------
 
